@@ -17,6 +17,7 @@ from ssl import CERT_NONE, CERT_OPTIONAL, CERT_REQUIRED, create_default_context
 
 
 from databricks.sql import common, USER_AGENT_NAME, __version__
+from databricks.sql.types import Row
 from databricks.sql.exc import *
 from databricks.sql.TCLIService import TCLIService, ttypes
 from builtins import range
@@ -504,7 +505,7 @@ class Cursor(common.DBAPICursor):
                 self._description.append((
                     col.columnName.decode('utf-8') if sys.version_info[0] == 2 else col.columnName,
                     type_code.decode('utf-8') if sys.version_info[0] == 2 else type_code,
-                    None, None, None, None, True
+                    None, None, None, None, None
                 ))
         return self._description
 
@@ -529,19 +530,74 @@ class Cursor(common.DBAPICursor):
         else:
             sql = operation % _escaper.escape_args(parameters)
 
-        self._reset_state()
-
-        self._state = self._STATE_RUNNING
-        _logger.info('%s', sql)
+        self.__prepare_state_for_execution()
 
         req = ttypes.TExecuteStatementReq(self._connection._sessionHandle,
                                           sql, runAsync=True)
-        _logger.debug(req)
         response = self._connection._make_request(self._connection._client.ExecuteStatement, req)
-        _check_status(response)
-        self._operationHandle = response.operationHandle
 
-        self.__wait_for_query_completion()
+        self.__handle_execute_response(response)
+
+    def catalogs(self):
+        """Get all available catalogs."""
+        self.__prepare_state_for_execution()
+
+        req = ttypes.TGetCatalogsReq(
+            sessionHandle=self._connection._sessionHandle)
+        response = self._connection._make_request(self._connection._client.GetCatalogs, req)
+
+        self.__handle_execute_response(response)
+
+    def schemas(self, catalog_name=None, schema_name=None):
+        """Get schemas corresponding to the catalog_name and schema_name.
+
+        Names can contain % wildcards.
+        """
+        self.__prepare_state_for_execution()
+
+        req = ttypes.TGetSchemasReq(
+            sessionHandle=self._connection._sessionHandle,
+            catalogName=catalog_name,
+            schemaName=schema_name)
+        response = self._connection._make_request(self._connection._client.GetSchemas, req)
+
+        self.__handle_execute_response(response)
+
+    def tables(self, catalog_name=None, schema_name=None, table_name=None, table_types=None):
+        """Get tables corresponding to the catalog_name, schema_name and table_name.
+        table_types is an optional list of table types to match e.g. "TABLE", "VIEW", etc.
+        Names can contain % wildcards.
+        """
+        if table_types is None:
+            table_types = []
+
+        self.__prepare_state_for_execution()
+
+        req = ttypes.TGetTablesReq(
+            sessionHandle=self._connection._sessionHandle,
+            catalogName=catalog_name,
+            schemaName=schema_name,
+            tableName=table_name,
+            tableTypes=table_types)
+        response = self._connection._make_request(self._connection._client.GetTables, req)
+
+        self.__handle_execute_response(response)
+
+    def columns(self, catalog_name=None, schema_name=None, table_name=None, column_name=None):
+        """Get columns corresponding to the catalog_name, schema_name, table_name and column_name.
+        Names can contain % wildcards.
+        """
+        self.__prepare_state_for_execution()
+        
+        req = ttypes.TGetColumnsReq(
+               sessionHandle=self._connection._sessionHandle,
+               catalogName=catalog_name,
+               schemaName=schema_name,
+               tableName=table_name,
+               columnName=column_name)
+        response = self._connection._make_request(self._connection._client.GetColumns, req)
+        
+        self.__handle_execute_response(response)
 
     def cancel(self):
         req = ttypes.TCancelOperationReq(
@@ -567,7 +623,10 @@ class Cursor(common.DBAPICursor):
         assert not response.results.rows, 'expected data in columnar format'
         columns = [_unwrap_column(col, col_schema[1]) for col, col_schema in
                    zip(response.results.columns, schema)]
-        new_data = list(zip(*columns))
+        
+        ResultRow = Row(*[c[0] for c in self.description])
+        new_data = [ResultRow(*r) for r in zip(*columns)]
+
         self._data += new_data
         # response.hasMoreRows seems to always be False, so we instead check the number of rows
         # https://github.com/apache/hive/blob/release-1.2.1/service/src/java/org/apache/hive/service/cli/thrift/ThriftCLIService.java#L678
@@ -575,6 +634,15 @@ class Cursor(common.DBAPICursor):
         if not new_data:
             self._state = self._STATE_FINISHED
 
+    def __prepare_state_for_execution(self):
+        self._reset_state()
+        self._state = self._STATE_RUNNING
+
+    def __handle_execute_response(self, response):
+        _check_status(response)
+        self._operationHandle = response.operationHandle
+        self.__wait_for_query_completion()
+
     def __wait_for_query_completion(self):
         state = ttypes.TOperationState.INITIALIZED_STATE
         while state in [ttypes.TOperationState.INITIALIZED_STATE, ttypes.TOperationState.PENDING_STATE, ttypes.TOperationState.RUNNING_STATE]:
